package main

import (
	"errors"
	"fmt"
	"io"
	"os"
	"path/filepath"
	"strings"
	"time"

	"git.milar.in/nyaanime/logic"
	"git.milar.in/nyaanime/model"
	"git.milar.in/nyaanime/parsers"
	"github.com/fatih/color"
)

func HandleFile(path string) (b *strings.Builder) {
	b = &strings.Builder{}
	b.WriteString(color.MagentaString("\nfile found: %s\n", filepath.Base(path)))

	for _, parser := range parsers.Parsers {
		parsedFile, ok := parser.FileParser(&parser, path)
		if !ok {
			b.WriteString(color.YellowString("\tnot parsable with parser '%s'\n", parser.Identity))
			continue
		}

		b.WriteString(color.GreenString("\tparsable with parser '%s'\n", parser.Identity))

		anime, err := logic.SearchAnimeByTitle(parsedFile.OriginalAnimeTitle)
		if err != nil {
			b.WriteString(color.YellowString("\tanime not found: '%s'\n", parsedFile.OriginalAnimeTitle))
			continue
		}

		parsedFile.Anime = anime
		HandleParsedFile(b, parsedFile)
		return
	}

	return
}

func HandleParsedFile(b *strings.Builder, parsedFile *model.ParsedFile) {
	newFilePrio := logic.NewFilePriority(parsedFile)
	oldFilePrio, animeEpNotExistLocally := logic.GetAnimeEpProps(parsedFile.AnimeEpisode())

	if !animeEpNotExistLocally || newFilePrio.Priority > oldFilePrio.Priority {
		go func(b *strings.Builder, parsedFile *model.ParsedFile) {
			if err := OrganizeAnimeEpisode(b, parsedFile); err != nil {
				b.WriteString(color.RedString("\terror: %s", err.Error()))
			}
		}(b, parsedFile)
	}
}

func OrganizeAnimeEpisode(b *strings.Builder, parsedFile *model.ParsedFile) error {
	start := time.Now()

	oldFile := filepath.Join(DownloadPath, parsedFile.File)
	newFile := logic.GetAnimeEpFilepath(parsedFile.AnimeEpisode(), filepath.Ext(parsedFile.File))
	lockFile := logic.GetAnimeEpFilepath(parsedFile.AnimeEpisode(), "lock")

	b.WriteString(color.BlueString("\tmove file\n\t    from: '%s'\n\t    to: '%s'\n", oldFile, newFile))

	if err := os.MkdirAll(filepath.Dir(newFile), os.ModePerm); err != nil {
		return err
	}

	if err := os.Chown(filepath.Dir(newFile), Uid, Gid); err != nil {
		return err
	}

	if _, err := os.Stat(newFile); !errors.Is(err, os.ErrNotExist) {
		fmt.Fprintln(os.Stderr, "file already exists:", newFile)
		return err
	}

	inputFile, err := os.Open(oldFile)
	if err != nil {
		return err
	}
	defer inputFile.Close()

	outputFile, err := os.Create(newFile)
	if err != nil {
		return err
	}
	defer outputFile.Close()

	if err := os.Chown(newFile, Uid, Gid); err != nil {
		return err
	}

	_, err = io.Copy(outputFile, inputFile)
	if err != nil {
		return err
	}

	if err = os.Remove(oldFile); err != nil {
		return err
	}

	if err = os.Remove(lockFile); err != nil {
		return err
	}

	b.WriteString(color.BlueString("\tdone (took %s)\n", time.Since(start)))

	return nil
}
